////////////////////////////////////////////////////////////////////////////////
//                            CBitmap32
// File name: Bitmap32.cpp
// Author: Sergey Krivulya (Ce p) - KSerg
// e-mail: Sergey_Krivulya@UkrPost.Net
// Date: 21 07 2004
//
// Description:     ... ...
//                  - - SetPixel,
//                  HDC,    BITAMP'...
//              ,    SetPixel' 
//                 API - ::SetPixel()
////////////////////////////////////////////////////////////////////////////////
#include "stdafx.h"
#include "Bitmap32.h"
#include ".\Lib\CommonLib.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

#define FixColor(color) RGB(GetBValue(color),GetGValue(color),GetRValue(color))

////////////////////////////////////////////////////////////////////////////////
//                              implementation
////////////////////////////////////////////////////////////////////////////////

CBitmap32::CBitmap32() :
   m_hBmp  (NULL),
   m_pvData(NULL)
{
   m_Size.cx = m_Size.cy = 0;
}

CBitmap32::CBitmap32(HDC hDC, SIZE size) :
   m_hBmp  (NULL),
   m_pvData(NULL)
{
   m_Size.cx = m_Size.cy = 0;
   Load(hDC, size);
}

CBitmap32::CBitmap32(HDC hCDC) :
   m_hBmp  (NULL),
   m_pvData(NULL)
{
   m_Size.cx = m_Size.cy = 0;
   Load(hCDC);
}

CBitmap32::CBitmap32(HWND hWnd, bool clientRect) :
   m_hBmp  (NULL),
   m_pvData(NULL)
{
   m_Size.cx = m_Size.cy = 0;
   Load(hWnd);
}

CBitmap32::CBitmap32(HBITMAP hBmp) :
   m_hBmp  (NULL),
   m_pvData(NULL)
{
   m_Size.cx = m_Size.cy = 0;
   Load(hBmp);
}

CBitmap32::CBitmap32(LPCTSTR szFile) :
   m_hBmp  (NULL),
   m_pvData(NULL)
{
   m_Size.cx = m_Size.cy = 0;
   Load(szFile);
}

CBitmap32::CBitmap32(const CBitmap32 &Bmp) :
   m_hBmp  (NULL),
   m_pvData(NULL)
{
   m_Size.cx = m_Size.cy = 0;
   Load(Bmp);
}

CBitmap32::CBitmap32(SIZE size, COLORREF colorFill) {
   Load(size, colorFill);
}

CBitmap32::~CBitmap32()
{
   Clean();
}

void CBitmap32::Clean() {
   if (m_hBmp) {
      ::DeleteObject(m_hBmp);
   }
   m_hBmp   = NULL;
   m_pvData = NULL;
   m_Size.cx = m_Size.cy = 0;
}

BOOL CBitmap32::Load(HDC hCDC)
{
   Clean();
   m_Size = SizeBitmap(hCDC);
   if ((m_Size.cx <= 0) || (m_Size.cy <= 0)) return FALSE;

   BITMAPINFO bih = {{// create DIB Section
      sizeof(BITMAPINFOHEADER), // bih.bmiHeader.biSize         
      m_Size.cx,                // bih.bmiHeader.biWidth        
      m_Size.cy,                // bih.bmiHeader.biHeight       
      1,                        // bih.bmiHeader.biPlanes       
      32,                       // bih.bmiHeader.biBitCount     
      BI_RGB,                   // bih.bmiHeader.biCompression  
      0,                        // bih.bmiHeader.biSizeImage    
      0,                        // bih.bmiHeader.biXPelsPerMeter
      0,                        // bih.bmiHeader.biYPelsPerMeter
      0,                        // bih.bmiHeader.biClrUsed      
      0                         // bih.bmiHeader.biClrImportant 
   },
   {
      {
         0, // bmiColors[1].rgbBlue
         0, // bmiColors[1].rgbGreen
         0, // bmiColors[1].rgbRed
         0  // bmiColors[1].rgbReserved
      }
   }};

   // create valid handle bitmap
   {
      HDC hTempDC = ::GetDC(NULL);
      m_hBmp = ::CreateDIBSection(hTempDC, &bih, DIB_RGB_COLORS, &m_pvData, NULL, 0);
      ::ReleaseDC(NULL, hTempDC);
   }

   // Copy the bits into our 32 bit dib..
   {
      HDC hTempDC = ::CreateCompatibleDC(hCDC);
      HBITMAP hOldBmp2 = (HBITMAP)::SelectObject(hTempDC, m_hBmp);
      BOOL bRes = ::BitBlt(hTempDC, 0,0,m_Size.cx,m_Size.cy, hCDC, 0,0, SRCCOPY);
      ::SelectObject(hTempDC, hOldBmp2);
      ::DeleteDC(hTempDC);
   }

   return TRUE;
}

BOOL CBitmap32::Load(HDC hDC, SIZE size)
{
   HDC hCDC = ::CreateCompatibleDC(hDC);
   HBITMAP hBmp = ::CreateCompatibleBitmap(hDC, size.cx, size.cy);
   HBITMAP hBmpOld = (HBITMAP)::SelectObject(hCDC, hBmp);
   ::BitBlt(hCDC, 0,0,size.cx,size.cy, hDC, 0,0, SRCCOPY);

   BOOL bRes = Load(hCDC);

   ::DeleteObject(::SelectObject(hCDC, hBmpOld));
   ::DeleteDC(hCDC);

   return bRes;
}

BOOL CBitmap32::Load(HWND hWnd, bool clientRect)
{
   HDC hDC = clientRect ? ::GetDC(hWnd) : ::GetWindowDC(hWnd);
   HDC hCDC = ::CreateCompatibleDC(hDC);
   SIZE size = clientRect ? ::GetClientRect(hWnd) : GetWindowRect(hWnd);
   HBITMAP hBmp = ::CreateCompatibleBitmap(hDC, size.cx, size.cy);
   HBITMAP hBmpOld = (HBITMAP)::SelectObject(hCDC, hBmp);
   ::BitBlt(hCDC, 0,0,size.cx,size.cy, hDC, 0,0, SRCCOPY);
   BOOL bRes = Load(hCDC);
   ::DeleteObject(::SelectObject(hCDC, hBmpOld));
   ::DeleteDC(hCDC);
   ::ReleaseDC(hWnd, hDC);
   return bRes;
}

BOOL CBitmap32::Load(HBITMAP hBmp)
{
   HDC hCDC = ::CreateCompatibleDC(NULL);
   HBITMAP hBmpOld = (HBITMAP)::SelectObject(hCDC, hBmp);
   BOOL bRes = Load(hCDC);
   ::SelectObject(hCDC, hBmpOld);
   ::DeleteDC(hCDC);
   return bRes;
}

BOOL CBitmap32::Load(LPCTSTR szFile)
{
   if (!szFile || !szFile[0]) return FALSE;
   HBITMAP hBmp  = (HBITMAP)::LoadImage(NULL, szFile, IMAGE_BITMAP, 0, 0, LR_LOADFROMFILE | LR_DEFAULTCOLOR);
   BOOL bRes = Load(hBmp);
   ::DeleteObject(hBmp);
   return bRes;
}

BOOL CBitmap32::Load(const CBitmap32 &Bmp)
{
   return Load((HBITMAP)Bmp);
}

BOOL CBitmap32::Load(SIZE size, COLORREF colorFill)
{
   HDC hCDC = ::CreateCompatibleDC(NULL);
   HBITMAP hBmp = ::CreateBitmap(size);
   HBITMAP hBmpOld = (HBITMAP)::SelectObject(hCDC, hBmp);
   HBRUSH hBrush = ::CreateSolidBrush(colorFill);
   HBRUSH hBrushOld = (HBRUSH)::SelectObject(hCDC, hBrush);
   ::PatBlt(hCDC, 0,0,size.cx,size.cy, PATCOPY);

   BOOL bRes = Load(hCDC);

   ::DeleteObject(::SelectObject(hCDC, hBrushOld));
   ::DeleteObject(::SelectObject(hCDC, hBmpOld  ));
   ::DeleteDC(hCDC);

   return bRes;
}

COLORREF CBitmap32::GetPixel_NotSafe(int x, int y) const {
   return FixColor(((DWORD*)m_pvData)[((m_Size.cy-1-y)*m_Size.cx)+x]);
}

COLORREF CBitmap32::GetPixel(int x, int y) const {
   if ((x<0) ||
       (y<0) ||
       (x>=m_Size.cx) ||
       (y>=m_Size.cy)) return CLR_INVALID;
   if (!m_pvData) return CLR_INVALID;
   return GetPixel_NotSafe(x, y);
}

void CBitmap32::SetPixel_NotSafe(int x, int y, COLORREF color) {
   ((DWORD*)m_pvData)[((m_Size.cy-1-y)*m_Size.cx)+x] = FixColor(color);
}

BOOL CBitmap32::SetPixel(int x, int y, COLORREF color) {
   if ((x<0) ||
       (y<0) ||
       (x>=m_Size.cx) ||
       (y>=m_Size.cy)) return FALSE;
   if (!m_pvData) return FALSE;
   SetPixel_NotSafe(x, y, color);
   return TRUE;
}

BOOL CBitmap32::BitBlt(
   HDC   hDCDest, // handle to destination device context
   RECT  rcDst,   // destination rectangle
   POINT pnSrc,   // xy-coordinates of source rectangles upper-left corner
   DWORD dwRop    // raster operation code
) const {
   if ((m_Size.cx <= pnSrc.x) ||
       (m_Size.cy <= pnSrc.y)) return FALSE;

   HDC hCDC= ::CreateCompatibleDC(hDCDest);
   HBITMAP hBmpOld = (HBITMAP)::SelectObject(hCDC, m_hBmp);

   BOOL bRes = ::BitBlt(
      hDCDest,
      rcDst.left, rcDst.top, rcDst.right-rcDst.left, rcDst.bottom-rcDst.top,
      hCDC,
      pnSrc.x, pnSrc.y,
      dwRop
   );

   ::SelectObject(hCDC, hBmpOld);
   ::DeleteDC(hCDC);

   return bRes;
}

BOOL CBitmap32::StretchBlt(
   HDC   hDCDest, // handle to destination device context
   RECT  rcDst,   // destination rectangle
   RECT  rcSrc,   // source rectangle
   DWORD dwRop    // raster operation code
) const {
   if ((m_Size.cx <= rcSrc.left) ||
       (m_Size.cy <= rcSrc.top)) return FALSE;

   HDC hCDC= ::CreateCompatibleDC(hDCDest);
   HBITMAP hBmpOld = (HBITMAP)::SelectObject(hCDC, m_hBmp);

   BOOL bRes = ::StretchBlt(
      hDCDest,
      rcDst.left, rcDst.top, rcDst.right-rcDst.left, rcDst.bottom-rcDst.top,
      hCDC,
      rcSrc.left, rcSrc.top, rcSrc.right-rcSrc.left, rcSrc.bottom-rcSrc.top,
      dwRop
   );

   ::SelectObject(hCDC, hBmpOld);
   ::DeleteDC(hCDC);

   return bRes;
}

void DrawMaskedBitmap(HDC hDC, HBITMAP hBmp,
                      int xD, int yD,
                      int wD, int hD,
                      int wS, int hS, COLORREF transparentColor)
{
   COLORREF oldBkColor = SetBkColor(hDC, RGB(255, 255, 255));
	COLORREF oldTextColor = SetTextColor(hDC, RGB(0, 0, 0));

	HDC hCDC = CreateCompatibleDC(NULL);
   HBITMAP hBmpMask = CreateMask(hBmp, transparentColor);
	HBITMAP hOldBmp  = (HBITMAP)SelectObject(hCDC, hBmp);
	StretchBlt(hDC, xD, yD, wD, hD, hCDC, 0, 0, wS, hS, DSx);
	SelectObject(hCDC, hBmpMask);
	StretchBlt(hDC, xD, yD, wD, hD, hCDC, 0, 0, wS, hS, DSna);
	SelectObject(hCDC, hBmp);
	StretchBlt(hDC, xD, yD, wD, hD, hCDC, 0, 0, wS, hS, DSx);
	SelectObject(hCDC, hOldBmp);
   DeleteDC(hCDC);
   DeleteObject(hBmpMask);

	SetBkColor(hDC, oldBkColor);
	SetTextColor(hDC, oldTextColor);
}

