////////////////////////////////////////////////////////////////////////////////
//                            CBitmap32
// File name: Bitmap32.h
// Author: Sergey Krivulya (Ce p) - KSerg
// e-mail: Sergey_Krivulya@UkrPost.Net
// Date: 21 07 2004
//
// Description: see Bitmap32.cpp
////////////////////////////////////////////////////////////////////////////////
#ifndef __FILE__BITMAP32__
#define __FILE__BITMAP32__

#if _MSC_VER > 1000
   #pragma once
#endif // _MSC_VER > 1000

// raster operation code
#define DSx  0x00660046L
#define DSna 0x00220326L

class CBitmap32  
{
private:
   HBITMAP m_hBmp;
   SIZE    m_Size;
   VOID   *m_pvData; // pointer to the bitmaps bit values
public:
   void Clean();
   CBitmap32();
   CBitmap32(HWND    hWnd, bool clientRect = false);
   CBitmap32(HDC     hDC, SIZE size);
   CBitmap32(HDC     hCDC        ); // only for compatible device context
   CBitmap32(HBITMAP hBmp        ); // hBmp       
   CBitmap32(LPCTSTR szFile      );
   CBitmap32(const CBitmap32 &Bmp); //  
   CBitmap32(SIZE size, COLORREF colorFill=0x00FFFFFF); //  BMP      

  ~CBitmap32();

   BOOL Load(HDC     hCDC        ); // only for compatible device context
   BOOL Load(HDC     hDC, SIZE size);
   BOOL Load(HWND    hWnd, bool clientRect = true);
   BOOL Load(HBITMAP hBmp        ); // hBmp cannot associate wicth device context (hBmp       )
   BOOL Load(LPCTSTR szFile      );
   BOOL Load(const CBitmap32 &Bmp);
   BOOL Load(SIZE size, COLORREF colorFill);

   operator HBITMAP() const {return m_hBmp;};
   BOOL operator =(const CBitmap32 &Bmp) {return Load(Bmp);}

   COLORREF GetPixel(int x,   int y) const;
   COLORREF GetPixel(const POINT &p) const {return GetPixel(p.x, p.y);}
   COLORREF GetPixel_NotSafe(int x, int y) const;
   COLORREF GetPixel_NotSafe(const POINT &p) const {return GetPixel_NotSafe(p.x, p.y);}
   BOOL SetPixel(int x,   int y, COLORREF color);
   BOOL SetPixel(const POINT &p, COLORREF color) {return SetPixel(p.x, p.y, color);}
   void SetPixel_NotSafe(int x, int y, COLORREF color);
   void SetPixel_NotSafe(const POINT &p, COLORREF color) {SetPixel_NotSafe(p.x, p.y, color);}

   const SIZE& Size() const {return m_Size;}

   BOOL BitBlt(
      HDC   hDCDest, // handle to destination device context
      RECT  rcDst,   // destination rectangle
      POINT pnSrc,   // xy-coordinates of source rectangles upper-left corner
      DWORD dwRop    // raster operation code
   ) const;
  
   BOOL StretchBlt(
      HDC   hDCDest, // handle to destination device context
      RECT  rcDst,   // destination rectangle
      RECT  rcSrc,   // source rectangle
      DWORD dwRop    // raster operation code
   ) const;
};

void DrawMaskedBitmap(HDC hDC, HBITMAP hBmp,
                      int xD, int yD,
                      int wD, int hD,
                      int wS, int hS,
                      COLORREF transparentColor = -1);

#endif // __FILE__BITMAP32__
