////////////////////////////////////////////////////////////////////////////////
//                            CColor
// File name: Color.h
// Author: Sergey Krivulya (Ce p) - KSerg
// e-mail: Sergey_Krivulya@UkrPost.Net
// Date: 21 07 2004
//
// Description:    -    COLORREF     ...
////////////////////////////////////////////////////////////////////////////////
#ifndef __FILE__COLOR__
#define __FILE__COLOR__

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

class CColor
{
   COLORREF m_color;
public:
   CColor(): m_color(0) {}
   CColor(COLORREF color): m_color(color) {}
   CColor(const CColor &color): m_color(color.Color()) {}
   CColor(BYTE r, BYTE g, BYTE b): m_color(RGB(r,g,b)) {}
   // read
   COLORREF Color() const {return m_color;}
   BYTE R() const {return GetRValue(m_color);}
   BYTE G() const {return GetGValue(m_color);}
   BYTE B() const {return GetBValue(m_color);}
   operator COLORREF() const {return Color();}
   // write
   COLORREF operator = (const CColor &color) {return m_color=color.Color();}
   COLORREF operator = (COLORREF      color) {return m_color=color;}
   void Color(COLORREF color) {m_color = color;}
   void R(BYTE r) {m_color = ((m_color & 0x00FFFF00) |        r        );}
   void G(BYTE g) {m_color = ((m_color & 0x00FF00FF) | ( WORD(g) <<  8));}
   void B(BYTE b) {m_color = ((m_color & 0x0000FFFF) | (DWORD(b) << 16));}

   //     colFirst  colorFinal
   static CColor IntermediateColor(
      const CColor &colorFirst, //  
      const CColor &colorFinal, //  
      ULONG ulStep,             //         
      ULONG ulInterval          //  
   ) {
      if (colorFirst.Color() == colorFinal.Color()) return colorFirst;
      if (ulStep <= 0           ) return colorFirst;
      if (ulStep >= ulInterval-1) return colorFinal;
      return CColor(
         BYTE(colorFirst.R() + (colorFinal.R() - colorFirst.R()) * (float(ulStep)/float(ulInterval-1))),
         BYTE(colorFirst.G() + (colorFinal.G() - colorFirst.G()) * (float(ulStep)/float(ulInterval-1))),
         BYTE(colorFirst.B() + (colorFinal.B() - colorFirst.B()) * (float(ulStep)/float(ulInterval-1)))
      );
   }
};

#endif
