////////////////////////////////////////////////////////////////////////////////
//                            CFire
// File name: Fire.cpp
// Author: Sergey Krivulya (Ce p) - KSerg
// e-mail: Sergey_Krivulya@UkrPost.Net
// Date: 21 07 2004
//
// Description:    .
//                 :
// 1. FirstIteration -    
// 2. NextIteration  -    
//
//   ' v.1.0 (+sources)'  http://infobase.h1.ru 
//        ... 
////////////////////////////////////////////////////////////////////////////////
#include "stdafx.h"
#include "Fire.h"
#include ".\Lib\CommonLib.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

void GeneratePoint(POINT &p, CFire::EDirection eDirection, int iDirectionSize)
{
   int r = rand(7+max(0, iDirectionSize));
   switch (r)
   {
   case 0:        p.y--; break;
   case 1: p.x--; p.y--; break;
   case 2: p.x++;        break;
   case 3: p.x--; p.y++; break;
   case 4:        p.y++; break;
   case 5: p.x++; p.y++; break;
   case 6: p.x--;        break;
   case 7: p.x++; p.y--; break;
   default:
      switch (eDirection) {
         case CFire::DIRECTION__LEFT_TOP    : if (r&1) p.x--; else p.y--;      break;
         case CFire::DIRECTION__TOP         :                      p.y--;      break; //   i  
         case CFire::DIRECTION__RIGHT_TOP   : if (r&1) p.x++; else p.y--;      break;
         case CFire::DIRECTION__LEFT        :          p.x--;                  break;
         case CFire::DIRECTION__CENTER      : GeneratePoint(p, eDirection, 0); break;
         case CFire::DIRECTION__RIGHT       :          p.x++;                  break;
         case CFire::DIRECTION__LEFT_BOTTOM : if (r&1) p.x--; else p.y++;      break;
         case CFire::DIRECTION__BOTTOM      :                      p.y++;      break;
         case CFire::DIRECTION__RIGHT_BOTTOM: if (r&1) p.x++; else p.y++;      break;
      }
   }
}

void ReplaceNoBkColor( //       
   IN const CBitmap32 &BmpFrom,
   OUT CBitmap32 &BmpResult,
   IN COLORREF colorBk,
   IN COLORREF colorReplace
) {
   BmpResult.Load(BmpFrom.Size(), colorBk);
   for (int x=0; x<BmpFrom.Size().cx; x++)
      for (int y=0; y<BmpFrom.Size().cy; y++) {
         COLORREF color = BmpFrom.GetPixel_NotSafe(x,y);
         if (color != colorBk)
            BmpResult.SetPixel_NotSafe(x,y,colorReplace);
      }
}

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CFire::CFire() :
   m_iFPS(0)
{
}

CFire::~CFire() {
   Release();
}

void CFire::Release()
{
}

BOOL CFire::FirstIteration(
   LPCTSTR szFileBmpOriginal,
   LPCTSTR szFileBmpMask,
   COLORREF colorHot,     //   
   COLORREF colorCold,    //  
   int      iInterval,    //  
   int      iSubInterval, //    ( colorHot  colorCold)
   COLORREF colorBk       //   == CLR_INVALID,     m_BmpOriginal[0][0]
) {
   Release();
   const_cast<CBitmap32*>(&m_BmpMask)->Clean();
   const_cast<CBitmap32*>(&m_BmpMasked)->Clean();
   const_cast<CBitmap32*>(&m_BmpOriginal)->Load(szFileBmpOriginal);
   if (m_BmpOriginal.Size().cx == 0) return FALSE;

   // masked
   if (szFileBmpMask && szFileBmpMask[0]) {
      const_cast<CBitmap32*>(&m_BmpMask)->Load(szFileBmpMask); // load mask
      SIZE size = m_BmpMask.Size();
      if (size.cx) {
         COLORREF cT = m_BmpMask.GetPixel(0,0);
         //cT = (cT==0) ? 0xFFFFFF : cT;
         HBITMAP hBmpMask = ::CreateMask(m_BmpMask, cT); // convert mask to monohrome bmp
         const_cast<CBitmap32*>(&m_BmpMask)->Load(hBmpMask);
         BOOL bRes = ::DeleteObject(hBmpMask);

         size = m_BmpOriginal.Size();
         // create masked (fired) bitmap
         const_cast<CBitmap32*>(&m_BmpMasked)->Load(size, m_colorBk);
         for (int i=0; i<size.cx; i++) {
            for (int j=0; j<size.cy; j++) {
               switch (m_BmpMask.GetPixel(i,j)) {
               case CLR_INVALID:
               case 0xFFFFFF:
                  const_cast<CBitmap32*>(&m_BmpMasked)->SetPixel(
                     i,j,
                     m_BmpOriginal.GetPixel_NotSafe(i,j)
                  );
                  break;
               }
            }
         }
      }
   }

   if (m_BmpMasked.Size().cx == 0) {
      const_cast<CBitmap32*>(&m_BmpMasked)->Load(m_BmpOriginal);
   }

   m_colorBk = (colorBk != CLR_INVALID) ? colorBk : m_BmpOriginal.GetPixel_NotSafe(0,0);
   m_Gr.Create(
      colorHot,
      colorCold,
      m_colorBk,
      iInterval,
      iSubInterval
   );
   m_BmpFire.Load(m_BmpOriginal.Size(), m_colorBk);
   return TRUE;
}

BOOL CFire::NextIteration(
   int iGradientStep,
   HDC hDC,
   int xLeft, int yTop,
   bool bOnlyFlame,
   int iPercentTransparent,
   EDirection eDirection,
   int iDirectionSize
)
{
   if (m_BmpOriginal.Size().cx == 0) return FALSE;

   static DWORD dwTickPrev = ::GetTickCount();
   static int iFPS = 0;

   SIZE size = m_BmpOriginal.Size();

   int qqqX = 0*size.cx/25;
   int qqqY = 0*size.cy/25;
   for (int x=0; x<size.cx; x+=1+rand(qqqX))
      for (int y=0; y<size.cy; y+=1+rand(qqqY)) {
         bool bOriginalC1 = false;
         COLORREF c1 = m_BmpFire.GetPixel_NotSafe(x,y);
         if (c1 == m_colorBk) {
            bOriginalC1 = true;
            c1 = m_BmpMasked.GetPixel_NotSafe(x,y);
         }
         if (c1 != m_colorBk) {
            int idx = 0;
            if (!bOriginalC1) {
               idx = m_Gr.GetIndex(c1);
               if (idx == -1) {
                  idx = m_Gr.Length()-1;//0;//
               } else {
                  idx += 1+rand(iGradientStep); //    
               }
               if (c1 == m_Gr[m_Gr.Length()-1]) {//     
                  //m_BmpFire.SetPixel_NotSafe(x,y,m_colorBk); //     ( )
               }
            }

            POINT p= {x,y};
            GeneratePoint(p, eDirection, iDirectionSize);
            COLORREF c2 = m_BmpMasked.GetPixel(p);

            if ((c2!=CLR_INVALID) //   "" 
                && (idx<m_Gr.Length())
               )
            {
               COLORREF c3 = m_colorBk;
               if (c2==m_colorBk)                 //  ?
               {
                  c3 =  m_Gr[idx];                // .    -  .
               } else {                           //     
                  if (rand(rand(100))>rand(50)) { //       
                     c3 = m_Gr[idx];              //     
                  } else {
                     //c3 = m_colorBk;            //    -     
                     //const_cast<CBitmap32*>(&m_BmpMasked)->SetPixel_NotSafe(p, m_colorBk); //   ...
                  }
               }
               m_BmpFire.SetPixel_NotSafe(p, c3);
            }
         }
      }

   BOOL bRes;

   if (bOnlyFlame) {
      bRes = m_BmpFire.BitBlt(hDC, RECTEX(xLeft, yTop, xLeft+size.cx, yTop+size.cy), POINTEX(0,0), SRCCOPY);
   } else {
      HDC hCDC = ::CreateCompatibleDC(hDC);
      HBITMAP hBmp = ::CreateBitmap(size);
      HBITMAP hBmpOld = (HBITMAP)::SelectObject(hCDC, hBmp);
      if (iPercentTransparent == -1) {

         //bRes = ::BitBlt(hCDC, 0,0,size.cx,size.cy, hDC, xLeft, yTop, SRCCOPY);

         bRes &= m_BmpOriginal.BitBlt(hCDC, RECTEX(0, 0, size.cx, size.cy), POINTEX(0,0), SRCCOPY);
         ::DrawMaskedBitmap(hCDC, m_BmpFire, 0, 0, size.cx, size.cy, size.cx, size.cy, m_colorBk);
         bRes &= ::BitBlt(hDC, xLeft, yTop, size.cx, size.cy, hCDC, 0,0, SRCCOPY);

      } else {
         CBitmap32 BmpResult(size, m_colorBk);
         const CBitmap32 &Bmp1 = m_BmpOriginal;//m_BmpMasked;//
         const CBitmap32 &Bmp2 = m_BmpFire;
         ///////////////////
         const float fP1 = max(0,min(100, iPercentTransparent))/100.f;
         const float fP2 = 1.f-fP1;
         for (int x=0; x<size.cx; x++) {
            for (int y=0; y<size.cy; y++)
            {
               CColor   c1(Bmp1.GetPixel(x,y));
               CColor   c2(Bmp2.GetPixel(x,y));
               COLORREF c3;
               if (c1==m_colorBk) {c3 = c2;} else
               if (c2==m_colorBk) {c3 = c1;} else
               if (c1==c2       ) {c3 = c1;} else
               {
                  c3 = RGB(c1.R()*fP2+c2.R()*fP1,
                           c1.G()*fP2+c2.G()*fP1,
                           c1.B()*fP2+c2.B()*fP1);
               }
               BmpResult.SetPixel(x, y, c3);
            }
         }
         ///////////////////
         bRes = BmpResult.BitBlt(hDC, RECTEX(xLeft, yTop, xLeft+size.cx, yTop+size.cy), POINTEX(0,0), SRCCOPY);
      }
      HGDIOBJ hObj = ::SelectObject(hCDC, hBmpOld);
      bRes &= ::DeleteObject(hBmp);
      bRes &= ::DeleteDC(hCDC);
   }

   iFPS++;

   DWORD dwTick = ::GetTickCount();

   if ((dwTick-dwTickPrev)>=1000) {
      dwTickPrev = dwTick;
      m_iFPS = iFPS;
      iFPS = 0;
   }

   return bRes;
}

