////////////////////////////////////////////////////////////////////////////////
//                            CFire
// File name: Fire.h
// Author: Sergey Krivulya (Ce p) - KSerg
// e-mail: Sergey_Krivulya@UkrPost.Net
// Date: 21 07 2004
//
// Description:   .
////////////////////////////////////////////////////////////////////////////////
#ifndef __FILE__CFIRE__
#define __FILE__CFIRE__

#include "Color.h"
#include "Bitmap32.h"

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

class CFire
{
public:
   enum EDirection {
      DIRECTION__LEFT_TOP     = 0,
      DIRECTION__TOP          = 1,
      DIRECTION__RIGHT_TOP    = 2,
      DIRECTION__LEFT         = 3,
      DIRECTION__CENTER       = 4,
      DIRECTION__RIGHT        = 5,
      DIRECTION__LEFT_BOTTOM  = 6,
      DIRECTION__BOTTOM       = 7,
      DIRECTION__RIGHT_BOTTOM = 8
   };
   ////////////////////////////////////////////////
   class CGradient { //   -    colorHot  colorCold  colorBk
   private:
      COLORREF *m_pColor;
      int       m_iLen;
   public:
      int Length() const {return m_iLen;}
      friend CFire;
   private:
      void Release() {
         if (m_pColor) {
            delete [] m_pColor;
            m_pColor = NULL;
            m_iLen = 0;
         }
      }
      CGradient() : m_iLen(0), m_pColor(NULL) {}
     ~CGradient() { Release(); }

   public:
      void Create( //   -    colorHot  colorCold  colorBk
         COLORREF colorHot,    //   
         COLORREF colorCold,   //  
         COLORREF colorBk,     //  
         int      iInterval,   //   
         int      iSubInterval //    ( colorHot  colorCold)
      )
      {
         Release();
         m_pColor = new COLORREF[m_iLen = iInterval];
         int iPart = iSubInterval;
         int i=0;
         for (; i<iPart; i++) {
            m_pColor[i] = CColor::IntermediateColor(colorHot , colorCold, i, iPart);
         }
         iPart = m_iLen-iSubInterval;
         for (int j=0; j<iPart; j++, i++) {
            m_pColor[i] = CColor::IntermediateColor(colorCold, colorBk  , j, iPart);
         }
      }
      COLORREF operator[](int i) const {
         if (i<0      ) return m_pColor[0       ]; // CLR_INVALID; // 
         if (i>=m_iLen) return m_pColor[m_iLen-1]; // CLR_INVALID; // 
         return m_pColor[i];
      }
      int GetIndex(COLORREF c) const { //    
         int index = -1;
         for (int i=0; i<m_iLen; i++) {
            if (m_pColor[i] == c) {
               index = i;
               break;
            }
         }
         return index;
      }
   };
   ////////////////////////////////////////////////
public:
   const CBitmap32 m_BmpOriginal;
   const CBitmap32 m_BmpMask;   // 
   const CBitmap32 m_BmpMasked; //   ,  m_BmpOriginal  ...
private:
         CBitmap32 m_BmpFire;
   COLORREF m_colorBk;
   int m_iFPS;

   void Release();
public:
	CFire();
  ~CFire();

   CGradient m_Gr;

   BOOL FirstIteration(
      LPCTSTR szFileBmpOriginal,
      LPCTSTR szFileBmpMask,
      COLORREF colorHot,     //   
      COLORREF colorCold,    //  
      int      iInterval,    //  
      int      iSubInterval, //    ( colorHot  colorCold)
      COLORREF colorBk = CLR_INVALID //   == CLR_INVALID,     m_BmpOriginal[0][0]
   );
   BOOL NextIteration(
      int iGradientStep,
      HDC hDC,
      int xLeft,
      int yTop,
      bool bOnlyFlame,
      int iPercentTransparent = -1,
      EDirection eDirection = DIRECTION__CENTER,
      int iDirectionSize = 0
   );

   COLORREF GetColorBk() const {return m_colorBk;}
   int FPS() const {return m_iFPS;}
};

#endif // __FILE__CFIRE__
