////////////////////////////////////////////////////////////////////////////////
//                            CFireTestDlg
// File name: FireTestDlg.cpp
// Author: Sergey Krivulya (Ce p) - KSerg
// e-mail: Sergey_Krivulya@UkrPost.Net
// Date: 21 07 2004
//
// Description:    .
////////////////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "Bitmap32.h"
#include "FireTest.h"
#include "FireTestDlg.h"
#include "Color.h"
#include ".\Lib\CommonLib.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

#define DEFAULT_GRADIENT_LEN 80
#define MAX_SPIN_GRADIENT_PRECISION_VALUE 500
#define MAX_SPIN_GRADIENT_STEP 10
#define MAX_SPIN_TRANSPARENT   100
#define MAX_SPIN_DIRECTION     100

#define ID_EVENT_TIMER__FIRE_NEXT_ITERATION            111
#define ID_EVENT_TIMER__FIND_WINDOW_CHOOSE_COLOR_HOT   220
#define ID_EVENT_TIMER__FIND_WINDOW_CHOOSE_COLOR_COLD  221
#define ID_EVENT_TIMER__FIND_WINDOW_CHOOSE_COLOR_BK    222


//#define REDRAW_FLAME_ONTIMER
#ifndef REDRAW_FLAME_ONTIMER
   #define REDRAW_FLAME_ONPAINT
#endif

const static int iOffsetBmpX = 8;
const static int iOffsetBmpY = 160;



CString LoadBmpFile(HWND hWnd) {
   CString strFileName;
   OPENFILENAME OpenFileName;
   OpenFileName.lStructSize = sizeof(OPENFILENAME);
   OpenFileName.hwndOwner   = hWnd;
   OpenFileName.hInstance   = NULL;
   TCHAR szFilter[256] = TEXT("Windows Bitmap (*.bmp)\0*.bmp\0\0");
   OpenFileName.lpstrFilter = szFilter;
   OpenFileName.lpstrCustomFilter = NULL;
 //OpenFileName.nMaxCustFilter;
   OpenFileName.nFilterIndex   = 0;
   TCHAR szFile[MAX_PATH] = {0};
   OpenFileName.lpstrFile      = szFile;
   OpenFileName.nMaxFile       = MAX_PATH;
   OpenFileName.lpstrFileTitle = NULL;
 //OpenFileName.nMaxFileTitle;
   TCHAR szDir[MAX_PATH] = {0};
   GetCurrentDirectory(MAX_PATH, szDir);
   OpenFileName.lpstrInitialDir = szDir;
   OpenFileName.lpstrTitle      = TEXT("Open Image");
   OpenFileName.Flags           = OFN_EXPLORER | OFN_FILEMUSTEXIST | OFN_HIDEREADONLY | OFN_LONGNAMES | OFN_PATHMUSTEXIST;
 //OpenFileName.nFileOffset;
 //OpenFileName.nFileExtension;
   OpenFileName.lpstrDefExt = NULL;
 //OpenFileName.lCustData;
 //OpenFileName.lpfnHook;
 //OpenFileName.lpTemplateName;
   if (GetOpenFileName(&OpenFileName)) {
      strFileName = OpenFileName.lpstrFile;
   }
   return strFileName;
}

//  -      IDC_STATIC_GRADIENT_BITMAP
LRESULT CALLBACK CFireTestDlg::newWndProc_StaticGradientBMP(HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam)
{
   CFireTestDlg *This = (CFireTestDlg*)::GetWindowLong(hwnd, GWL_USERDATA);
   if (This != NULL) {
      switch (msg) {
      case WM_NCPAINT:
      case WM_ERASEBKGND: return -1;
      case WM_PAINT:
         // default redraw
         ::DefWindowProc(hwnd, msg, wParam, lParam);
         // my redraw
         if (This->m_Fire.m_Gr.Length()>0)
         {
            SIZE size = GetWindowSize(hwnd);
            HDC hCDC = ::CreateCompatibleDC(NULL);
            HBITMAP hBmp  = ::CreateBitmap(This->m_Fire.m_Gr.Length(), 1);
            HBITMAP hBmpOld = (HBITMAP)::SelectObject(hCDC, hBmp);

            for (int i=0; i<This->m_Fire.m_Gr.Length(); i++) {
               ::SetPixel(hCDC, i,0, This->m_Fire.m_Gr[i]);
            }

            HDC hDC  = ::GetWindowDC(hwnd);
            ::StretchBlt(hDC, 0,0,size.cx,size.cy, hCDC, 0,0,This->m_Fire.m_Gr.Length(),1, SRCCOPY);
            ::ReleaseDC(hwnd, hDC);

            ::SelectObject(hCDC, hBmpOld);
            ::DeleteObject(hBmp);
            ::DeleteDC(hCDC);
         }
         return 0;
      }
      return CallWindowProc((WNDPROC)This->defWndProc_StaticGradientBMP, hwnd, msg, wParam, lParam);
   }
   return ::DefWindowProc(hwnd, msg, wParam, lParam);
}

/////////////////////////////////////////////////////////////////////////////
// CFireTestDlg dialog

CFireTestDlg::CFireTestDlg(CWnd* pParent /*=NULL*/)
	: CDialog(CFireTestDlg::IDD, pParent),
   defWndProc_StaticGradientBMP(NULL)
{
	//{{AFX_DATA_INIT(CFireTestDlg)
	//}}AFX_DATA_INIT
	// Note that LoadIcon does not require a subsequent DestroyIcon in Win32
	m_hIcon = AfxGetApp()->LoadIcon(IDR_MAINFRAME);
}

void CFireTestDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CFireTestDlg)
	DDX_Control(pDX, IDC_SPIN_DIRECTION, m_SpinDirection);
	DDX_Control(pDX, IDC_RADIO_LT, m_RadioBtnDirection);
	DDX_Control(pDX, IDC_BUTTON_LOAD_BMP_MASKED, m_BtnBmpMasked);
	DDX_Control(pDX, IDC_BUTTON_LOAD_BMP_MASK, m_BtnBmpMask);
	DDX_Control(pDX, IDC_BUTTON_LOAD_BMP_ORIGINAL, m_BtnBmpOriginal);
	DDX_Control(pDX, IDC_CHECK_TRANSPARENT, m_CheckTransparent);
	DDX_Control(pDX, IDC_SPIN_TRANSPARENT, m_SpinTransparent);
	DDX_Control(pDX, IDC_CHECK_FIRE_ONLY, m_CheckFlameOnly);
	DDX_Control(pDX, IDC_SPIN_GRADIENT_STEP, m_SpinGradientStep);
	DDX_Control(pDX, IDC_CHECK_RESTART_CHANGES, m_CheckRestart);
	DDX_Control(pDX, IDC_SPIN_GRADIENT_PRECISION, m_SpinGradientPrecision);
	DDX_Control(pDX, IDC_STATIC_GRADIENT_BITMAP, m_StaticGradientBMP);
	DDX_Control(pDX, IDC_SLIDER_GRADIENT, m_SliderGradient);
	//}}AFX_DATA_MAP

}

BEGIN_MESSAGE_MAP(CFireTestDlg, CDialog)
	//{{AFX_MSG_MAP(CFireTestDlg)
	ON_WM_TIMER()
	ON_BN_CLICKED(IDC_BUTTON_LOAD_BMP_ORIGINAL, OnButton_LoadBmpOriginal)
	ON_WM_PAINT()
	ON_WM_HSCROLL()
	ON_EN_CHANGE(IDC_EDIT_GRADIENT_PRECISION, OnChange_EditGradientPrecision)
	ON_BN_CLICKED(IDC_STATIC_GRADIENT_BITMAP, OnClicked_StaticGradientBitmap)
	ON_BN_CLICKED(IDC_BUTTON_LOAD_BMP_MASK, OnButton_LoadBmpMask)
	ON_BN_CLICKED(IDC_CHECK_RESTART_CHANGES, OnCheck_RestartChanges)
	ON_EN_CHANGE(IDC_EDIT_GRADIENT_STEP, OnChange_EditGradientStep)
	ON_EN_CHANGE(IDC_EDIT_TRANSPARENT, OnChange_EditTransparent)
	ON_BN_CLICKED(IDC_CHECK_FIRE_ONLY, OnCheck_FireOnly)
	ON_BN_CLICKED(IDC_CHECK_TRANSPARENT, OnCheck_Transparent)
	ON_EN_CHANGE(IDC_EDIT_DIRECTION, OnChange_EditDirection)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CFireTestDlg message handlers

BOOL CFireTestDlg::OnInitDialog()
{
	CDialog::OnInitDialog();

	SetIcon(m_hIcon, TRUE);			// Set big icon
	SetIcon(m_hIcon, FALSE);		// Set small icon
	
                 //BBGGRR
   m_colorHot  = 0x00FFFF; // yellow
   m_colorCold = 0x0000FF; // red
   m_colorBk   = CLR_INVALID;

   m_SpinTransparent.SetRange(0, MAX_SPIN_TRANSPARENT);
   m_SpinTransparent.SetPos(MAX_SPIN_TRANSPARENT/2);

   m_SpinGradientPrecision.SetRange(10, MAX_SPIN_GRADIENT_PRECISION_VALUE);
   m_SpinGradientPrecision.SetPos(DEFAULT_GRADIENT_LEN);

   m_SpinDirection.SetRange(0, MAX_SPIN_DIRECTION);
   m_SpinDirection.SetPos(30);

   m_SliderGradient.SetRange(0, DEFAULT_GRADIENT_LEN, TRUE);
   m_SliderGradient.SetPos(DEFAULT_GRADIENT_LEN/2);
   m_SpinGradientStep.SetRange(1, MAX_SPIN_GRADIENT_STEP);
   m_SpinGradientStep.SetPos(4);
   m_fPercent = m_SliderGradient.GetPos()*100.f/DEFAULT_GRADIENT_LEN;
   OnCheck_RestartChanges();
   OnCheck_FireOnly();

   //  -      IDC_STATIC_GRADIENT_BITMAP
   ::SetWindowLong(m_StaticGradientBMP.m_hWnd, GWL_USERDATA, (LONG)this);
   defWndProc_StaticGradientBMP = (WNDPROC)::SetWindowLong(m_StaticGradientBMP.m_hWnd, GWL_WNDPROC, (LONG)newWndProc_StaticGradientBMP);

   m_strCaption = ::GetWindowText(m_hWnd);
   ::SendMessage(::GetDlgItem(m_hWnd, IDC_RADIO_T), BM_SETCHECK, BST_CHECKED, 0L);

   OnButton_LoadBmpOriginal();

   return TRUE;  // return TRUE  unless you set the focus to a control
}

void CFireTestDlg::OnTimer(UINT nIDEvent)
{
   switch (nIDEvent)
   {
   case ID_EVENT_TIMER__FIRE_NEXT_ITERATION:
      {
#ifdef REDRAW_FLAME_ONTIMER
         HDC hDC = ::GetDC(m_hWnd);
         DrawFlameOrMask(hDC);
         ::ReleaseDC(m_hWnd, hDC);
#else
   #ifdef REDRAW_FLAME_ONPAINT
         RECTEX rcBmp(
            max(m_Fire.m_BmpOriginal.Size().cx, m_Fire.m_BmpMask.Size().cx),
            max(m_Fire.m_BmpOriginal.Size().cy, m_Fire.m_BmpMask.Size().cy)
         );
         rcBmp.move(iOffsetBmpX, iOffsetBmpY);
         ::InvalidateRect(m_hWnd, &rcBmp, FALSE);
   #endif
#endif
      }
      break;
   case ID_EVENT_TIMER__FIND_WINDOW_CHOOSE_COLOR_HOT:
   case ID_EVENT_TIMER__FIND_WINDOW_CHOOSE_COLOR_COLD:
   case ID_EVENT_TIMER__FIND_WINDOW_CHOOSE_COLOR_BK:
      {
         HWND hWndColor = ::GetForegroundWindow();
         HWND hWndParent = ::GetParent(hWndColor);
         if (hWndParent == m_hWnd) {
            CString strCaption = ::GetWindowText(hWndColor);
            CString strAdd;
            switch (nIDEvent) {
               case ID_EVENT_TIMER__FIND_WINDOW_CHOOSE_COLOR_HOT:  strAdd = TEXT("Hot"); break;
               case ID_EVENT_TIMER__FIND_WINDOW_CHOOSE_COLOR_COLD: strAdd = TEXT("Cold"); break;
               case ID_EVENT_TIMER__FIND_WINDOW_CHOOSE_COLOR_BK:   strAdd = TEXT("Background"); break;
            }
            ::SetWindowText(hWndColor, strCaption + TEXT(" - ") + strAdd);
            KillTimer(nIDEvent);
         }
      }
      break;
   }
   CDialog::OnTimer(nIDEvent);
}

void CFireTestDlg::OnButton_LoadBmpOriginal() 
{
   m_strFileBmpOriginal = LoadBmpFile(m_hWnd);
   if (!m_strFileBmpOriginal.IsEmpty())
   {
      BOOL bRes = m_Fire.FirstIteration(
         m_strFileBmpOriginal,
         (m_strFileBmpMask.Empty(), m_strFileBmpMask),
         m_colorHot,
         m_colorCold,
         m_SliderGradient.GetRangeMax(),
         m_SliderGradient.GetPos()
      );
      if (bRes && (m_Fire.m_Gr.Length()>0)) {
         m_colorBk = m_Fire.m_Gr[m_Fire.m_Gr.Length()-1];
         KillTimer(ID_EVENT_TIMER__FIRE_NEXT_ITERATION);
         SetTimer(ID_EVENT_TIMER__FIRE_NEXT_ITERATION, 0, NULL);
         ::ShowWindow(::GetDlgItem(m_hWnd, IDC_BUTTON_LOAD_BMP_MASKED), SW_SHOW);
         ::ShowWindow(::GetDlgItem(m_hWnd, IDC_SLIDER_GRADIENT          ), SW_SHOW);
         ::ShowWindow(::GetDlgItem(m_hWnd, IDC_CHECK_RESTART_CHANGES    ), SW_SHOW);
         ::ShowWindow(::GetDlgItem(m_hWnd, IDC_STATIC_GRADIENT          ), SW_SHOW);
         ::ShowWindow(::GetDlgItem(m_hWnd, IDC_EDIT_GRADIENT_PRECISION  ), SW_SHOW);
         ::ShowWindow(::GetDlgItem(m_hWnd, IDC_STATIC_GRADIENT_PRECISION), SW_SHOW);
         ::ShowWindow(::GetDlgItem(m_hWnd, IDC_SPIN_GRADIENT_PRECISION  ), SW_SHOW);
         ::ShowWindow(::GetDlgItem(m_hWnd, IDC_STATIC_GRADIENT_BITMAP   ), SW_SHOW);
         ::ShowWindow(::GetDlgItem(m_hWnd, IDC_BUTTON_LOAD_BMP_MASK     ), SW_SHOW);
         ::ShowWindow(::GetDlgItem(m_hWnd, IDC_STATIC_GRADIENT_STEP     ), SW_SHOW);
         ::ShowWindow(::GetDlgItem(m_hWnd, IDC_EDIT_GRADIENT_STEP       ), SW_SHOW);
         ::ShowWindow(::GetDlgItem(m_hWnd, IDC_SPIN_GRADIENT_STEP       ), SW_SHOW);
         ::ShowWindow(::GetDlgItem(m_hWnd, IDC_STATIC_FIRE              ), SW_SHOW);
         ::ShowWindow(::GetDlgItem(m_hWnd, IDC_CHECK_FIRE_ONLY          ), SW_SHOW);
         ::ShowWindow(::GetDlgItem(m_hWnd, IDC_CHECK_TRANSPARENT        ), SW_SHOW);
         ::ShowWindow(::GetDlgItem(m_hWnd, IDC_EDIT_TRANSPARENT         ), SW_SHOW);
         ::ShowWindow(::GetDlgItem(m_hWnd, IDC_SPIN_TRANSPARENT         ), SW_SHOW);
         ::ShowWindow(::GetDlgItem(m_hWnd, IDC_RADIO_LT                 ), SW_SHOW);
         ::ShowWindow(::GetDlgItem(m_hWnd, IDC_RADIO_T                  ), SW_SHOW);
         ::ShowWindow(::GetDlgItem(m_hWnd, IDC_RADIO_RT                 ), SW_SHOW);
         ::ShowWindow(::GetDlgItem(m_hWnd, IDC_RADIO_L                  ), SW_SHOW);
         ::ShowWindow(::GetDlgItem(m_hWnd, IDC_RADIO_C                  ), SW_SHOW);
         ::ShowWindow(::GetDlgItem(m_hWnd, IDC_RADIO_R                  ), SW_SHOW);
         ::ShowWindow(::GetDlgItem(m_hWnd, IDC_RADIO_LB                 ), SW_SHOW);
         ::ShowWindow(::GetDlgItem(m_hWnd, IDC_RADIO_B                  ), SW_SHOW);
         ::ShowWindow(::GetDlgItem(m_hWnd, IDC_RADIO_RB                 ), SW_SHOW);
         ::ShowWindow(::GetDlgItem(m_hWnd, IDC_STATIC_DIRECTION         ), SW_SHOW);
         ::ShowWindow(::GetDlgItem(m_hWnd, IDC_EDIT_DIRECTION           ), SW_SHOW);
         ::ShowWindow(::GetDlgItem(m_hWnd, IDC_SPIN_DIRECTION           ), SW_SHOW);
      }
      Invalidate();
   }
}

void CFireTestDlg::OnButton_LoadBmpMask() 
{
   CString strNewFileMask = LoadBmpFile(m_hWnd);
   if (!strNewFileMask.IsEmpty()) {
      m_strFileBmpMask = strNewFileMask;
      ApplyChanges(TRUE);
      Invalidate();
   }
}

void CFireTestDlg::OnPaint() 
{
	CPaintDC dc(this); // device context for painting
   RECTEX rcPaint(dc.m_ps.rcPaint);

#ifdef REDRAW_FLAME_ONPAINT
   // draw flame
   {
      RECTEX rcBmp(
         max(m_Fire.m_BmpOriginal.Size().cx, m_Fire.m_BmpMask.Size().cx),
         max(m_Fire.m_BmpOriginal.Size().cy, m_Fire.m_BmpMask.Size().cy)
      );
      rcBmp.move(iOffsetBmpX, iOffsetBmpY);
      // ,   rcPaint  rcBmp, ..    
      if (IntersectRect(rcPaint, rcBmp)) {
         DrawFlameOrMask(dc.m_hDC);
      }
   }
#endif
}

void CFireTestDlg::OnHScroll(UINT nSBCode, UINT nPos, CScrollBar* pScrollBar) 
{
   if (pScrollBar->m_hWnd == m_SliderGradient.m_hWnd)
   {
      static int iPos = -1;
      if (iPos != m_SliderGradient.GetPos())
      {  //    
         iPos = m_SliderGradient.GetPos();
         m_fPercent = iPos*100.f/m_SliderGradient.GetRangeMax();
         ApplyChanges(m_bRestart);
      }
   }

	CDialog::OnHScroll(nSBCode, nPos, pScrollBar);
}

void CFireTestDlg::OnChange_EditGradientPrecision() 
{
   if (m_SpinGradientPrecision.m_hWnd != NULL) {
      int iSpinPos = m_SpinGradientPrecision.GetPos();
      if (iSpinPos > MAX_SPIN_GRADIENT_PRECISION_VALUE) {
         m_SpinGradientPrecision.SetPos(iSpinPos &= 0xFFFEFFFF); //    - ...
      }
      if (m_Fire.m_Gr.Length()>0)
      {
         m_SliderGradient.SetRange(0, iSpinPos);
         m_SliderGradient.SetPos(int(m_fPercent*m_SliderGradient.GetRangeMax()/100.f));
         ApplyChanges(m_bRestart);
      }
   }
}

void CFireTestDlg::OnChange_EditGradientStep() 
{
   if (m_SpinGradientStep.m_hWnd != NULL) {
      int iSpinPos = m_SpinGradientStep.GetPos();
      if (iSpinPos > MAX_SPIN_GRADIENT_STEP) {
         m_SpinGradientStep.SetPos(iSpinPos &= 0xFFFEFFFF); //    - ...
      }
      ApplyChanges(m_bRestart);
   }
}

void CFireTestDlg::OnChange_EditTransparent() 
{
   if (m_SpinTransparent.m_hWnd != NULL) {
      int iSpinPos = m_SpinTransparent.GetPos();
      if (iSpinPos > MAX_SPIN_TRANSPARENT) {
         m_SpinTransparent.SetPos(iSpinPos &= 0xFFFEFFFF); //    - ...
      }
   }
}

void CFireTestDlg::OnChange_EditDirection() 
{
   if (m_SpinDirection.m_hWnd != NULL) {
      int iSpinPos = m_SpinDirection.GetPos();
      if (iSpinPos > MAX_SPIN_DIRECTION) {
         m_SpinDirection.SetPos(iSpinPos &= 0xFFFEFFFF); //    - ...
      }
   }
}

void CFireTestDlg::OnClicked_StaticGradientBitmap() 
{
   if (m_Fire.m_Gr.Length()>0)
   {
      POINTEX p;
      ::GetCursorPos(&p);
      ::ScreenToClient(m_StaticGradientBMP.m_hWnd, &p);
      SIZE size = ::GetWindowSize(m_StaticGradientBMP.m_hWnd);
      int iWhatColor = int(p.x/(size.cx/3.f)); // 0 - Hot Color; 1 - Coll Color; 2 - Bk Color
      iWhatColor = max(0, min(2, iWhatColor));

      static COLORREF custColors[16] = {
         0x000000, 0x0000FF, 0x00FF00, 0x00FFFF, 0xFF0000, 0xFF00FF, 0xFFFF00, 0xFFFFFF,
         0xFFF000, 0x00007F, 0x007F00, 0x007F7F, 0x7F0000, 0x7F007F, 0x7F7F00, 0x7F7F7F
      };
      CHOOSECOLOR sChooseColor;
      sChooseColor.lStructSize  = sizeof(CHOOSECOLOR);
      sChooseColor.hwndOwner    = m_hWnd;
      sChooseColor.hInstance    = (HWND)AfxGetApp()->m_hInstance;
      sChooseColor.rgbResult    = (iWhatColor==0) ? m_colorHot : ((iWhatColor==1) ? m_colorCold : m_colorBk);
      sChooseColor.lpCustColors = custColors;
      sChooseColor.Flags        = CC_RGBINIT | CC_FULLOPEN | CC_SOLIDCOLOR;
    //sChooseColor.lCustData;
    //sChooseColor.lpfnHook;
    //sChooseColor.lpTemplateName;
      SetTimer(ID_EVENT_TIMER__FIND_WINDOW_CHOOSE_COLOR_HOT + iWhatColor, 0, NULL);
      KillTimer(ID_EVENT_TIMER__FIRE_NEXT_ITERATION);
      if (ChooseColor(&sChooseColor)) {
         switch (iWhatColor) {
         case  0: m_colorHot  = sChooseColor.rgbResult; ApplyChanges(m_bRestart); break;
         case  1: m_colorCold = sChooseColor.rgbResult; ApplyChanges(m_bRestart); break;
         default: m_colorBk   = sChooseColor.rgbResult; ApplyChanges(true);       break;
         }
         
      }
      SetTimer(ID_EVENT_TIMER__FIRE_NEXT_ITERATION, 0, NULL);
   }
}

void CFireTestDlg::OnCheck_RestartChanges() 
{
   m_bRestart = !!m_CheckRestart.GetCheck();
}

void CFireTestDlg::ApplyChanges(BOOL bRestart) {
   if (m_Fire.m_Gr.Length()>0)
   {
      if (bRestart) {
         m_Fire.FirstIteration(
            m_strFileBmpOriginal,
            m_strFileBmpMask,
            m_colorHot,
            m_colorCold,
            m_SliderGradient.GetRangeMax(),
            m_SliderGradient.GetPos(),
            m_colorBk
         );
      } else {
         m_Fire.m_Gr.Create(m_colorHot, m_colorCold, m_Fire.GetColorBk(), m_SliderGradient.GetRangeMax(), m_SliderGradient.GetPos());
      }
      m_StaticGradientBMP.Invalidate();
   }
}

void CFireTestDlg::OnCheck_FireOnly()
{
   bool bValue = !m_CheckFlameOnly.GetCheck();
   ::EnableWindow(::GetDlgItem(m_hWnd, IDC_CHECK_TRANSPARENT), bValue);
   bValue &= !!m_CheckTransparent.GetCheck();
   ::EnableWindow(::GetDlgItem(m_hWnd, IDC_EDIT_TRANSPARENT ), bValue);
   ::EnableWindow(::GetDlgItem(m_hWnd, IDC_SPIN_TRANSPARENT ), bValue);
}

void CFireTestDlg::OnCheck_Transparent() 
{
   OnCheck_FireOnly();
}

void CFireTestDlg::DrawFlameOrMask(HDC hDC)
{
   BOOL bCursorOverButtonBmpOriginal = false,
        bCursorOverButtonBmpMask     = false,
        bCursorOverButtonBmpMasked   = false;
   if (m_hWnd == ::GetForegroundWindow())
   {
      POINT p; ::GetCursorPos(&p);
      RECT rcOrig = ::GetWindowRect(m_BtnBmpOriginal.m_hWnd);
      RECT rcMask = ::GetWindowRect(m_BtnBmpMask    .m_hWnd);
      RECT rcMskd = ::GetWindowRect(m_BtnBmpMasked  .m_hWnd);
      bCursorOverButtonBmpOriginal = ((p.x>=rcOrig.left) && (p.x<rcOrig.right) && (p.y>=rcOrig.top) && (p.y<rcOrig.bottom));
      bCursorOverButtonBmpMask     = ((p.x>=rcMask.left) && (p.x<rcMask.right) && (p.y>=rcMask.top) && (p.y<rcMask.bottom) && m_Fire.m_BmpMask.Size().cx);
      bCursorOverButtonBmpMasked   = ((p.x>=rcMskd.left) && (p.x<rcMskd.right) && (p.y>=rcMskd.top) && (p.y<rcMskd.bottom) && m_BtnBmpMasked.IsWindowVisible());
   }

   CString strNewCaption = m_strCaption;
   if (bCursorOverButtonBmpOriginal) {
      m_Fire.m_BmpOriginal.BitBlt(hDC, RECTEX(m_Fire.m_BmpOriginal.Size()).move(iOffsetBmpX, iOffsetBmpY), POINTEX(0,0), SRCCOPY);
   } else
   if (bCursorOverButtonBmpMask) {
      m_Fire.m_BmpMask    .BitBlt(hDC, RECTEX(m_Fire.m_BmpMask    .Size()).move(iOffsetBmpX, iOffsetBmpY), POINTEX(0,0), SRCCOPY);
   } else
   if (bCursorOverButtonBmpMasked) {
      m_Fire.m_BmpMasked  .BitBlt(hDC, RECTEX(m_Fire.m_BmpMasked  .Size()).move(iOffsetBmpX, iOffsetBmpY), POINTEX(0,0), SRCCOPY);
   } else {
      m_Fire.NextIteration(
         m_SpinGradientStep.GetPos(),
         hDC,
         iOffsetBmpX,iOffsetBmpY,
         !!m_CheckFlameOnly.GetCheck(),
         !!m_CheckTransparent.GetCheck() ? m_SpinTransparent.GetPos() : -1,
         CFire::EDirection(GetCheckedRadioButton(IDC_RADIO_LT, IDC_RADIO_RB)-IDC_RADIO_LT),
         m_SpinDirection.GetPos()
      );
      CString strFPS; strFPS.Format(TEXT(" - FPS(%i)"), m_Fire.FPS());
      strNewCaption += strFPS;
   }
   SetWindowText(strNewCaption);
}
